package com.board.spring_security.config;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.configuration.AuthenticationConfiguration;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.provisioning.JdbcUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.access.AccessDeniedHandler;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.security.web.authentication.rememberme.JdbcTokenRepositoryImpl;
import org.springframework.security.web.authentication.rememberme.PersistentTokenRepository;

import com.board.spring_security.security.CustomAccessDeniedHandler;
import com.board.spring_security.security.LoginSuccessHandler;

import lombok.extern.slf4j.Slf4j;

@SuppressWarnings("deprecation")
@Slf4j
@Configuration
@EnableWebSecurity
@EnableGlobalMethodSecurity(prePostEnabled = true, securedEnabled = true)
public class SecurityConfig {

    @Autowired
    private DataSource dataSource;    
    
    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        log.info("스프링 시큐리티 설정...");
        

        http.authorizeRequests(authorize -> authorize
                                .requestMatchers("/**").permitAll()
                                .requestMatchers("/css/**", "/js/**", "/img/**").permitAll()
                                .requestMatchers("/user/**").hasAnyRole("USER", "ADMIN")
                                .requestMatchers("/admin/**").hasAnyRole("ADMIN")
                                .anyRequest().authenticated()
                                );
        
        
        http.formLogin( form -> form
                                .defaultSuccessUrl("/")
                                .loginPage("/login")
                                // .loginProcessingUrl("/loginPro")
                                .usernameParameter("id")
                                .passwordParameter("pw")
                                .successHandler(authenticationSuccessHandler())
                                .permitAll()
        );

        http.logout( logout -> logout
                            .logoutSuccessUrl("/login")
                            .logoutUrl("/logout")
                            .invalidateHttpSession(true)
                            .permitAll()
                    );

        http.rememberMe(remember -> remember
                                .key("kdh")
                                .tokenRepository(tokenRepository())
                                .tokenValiditySeconds(60 * 60 * 24 * 7)
                        );

        http.exceptionHandling( exception -> exception
                                            //.accessDeniedPage("/exception")
                                            .accessDeniedHandler(accessDeniedHandler())
                                );
        

        return http.build();
    }

    @Bean
    public PasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder(); 
    }

    @Bean
    public UserDetailsService customUserDetailsService() {
        JdbcUserDetailsManager userDetailsManager = new JdbcUserDetailsManager(dataSource);
        userDetailsManager.setUsersByUsernameQuery("SELECT user_id as username, user_pw as password, enabled"
                                                 + " FROM \"TB_USER\""
                                                 + " WHERE user_id = ?");
        userDetailsManager.setAuthoritiesByUsernameQuery("SELECT user_id as username, auth as authority"
                                                        +" FROM \"TB_AUTH\""
                                                        +" WHERE user_id = ?");

        return userDetailsManager;
    }

    @Bean
    public PersistentTokenRepository tokenRepository() {
        JdbcTokenRepositoryImpl repositoryImpl = new JdbcTokenRepositoryImpl();
        repositoryImpl.setDataSource(dataSource);

        return repositoryImpl;
    }

    @Bean
    public AuthenticationSuccessHandler authenticationSuccessHandler() {
        return new LoginSuccessHandler();
    }


    @Bean
    public AuthenticationManager authenticationManager(AuthenticationConfiguration authenticationConfiguration) throws Exception {
        return authenticationConfiguration.getAuthenticationManager();
    }

    @Bean
    public AccessDeniedHandler accessDeniedHandler() {
        return new CustomAccessDeniedHandler();
    }
}
